@@ -7,10 +7,13 @@ __ALL__ = ["Agent"]
 class Agent(object):
     def __init__(self, app_name: str, key_pair: key_pair_format, aes_key: bytes, hmac_key: bytes) -> None:
         self.__hmac = lambda _: mac(_, hmac_key)
-        self.__aese = lambda _: AESCipher(aes_key).encrypt(_)
-        self.__aesd = lambda _: AESCipher(aes_key).decrypt(_)
+        self.__skey = lambda: bytes(randb(len(aes_key))[i] ^ aes_key[i] for i in range(len(aes_key)))
+        self.__aese = lambda _, __: AESCipher(_).encrypt(__)
+        self.__aesd = lambda _, __: AESCipher(
+            EasyRSA(private_key=key_pair["private_key"]).decrypt(_)
+        ).decrypt(__)
+        self.__rsae = lambda _: EasyRSA(public_key=key_pair["public_key"]).encrypt(_)
         self.__app_name = self.__hmac(app_name)
-        self.__key_pair = key_pair
 
     @staticmethod
     def __check():
@@ -24,9 +27,11 @@ class Agent(object):
 
     def __encrypt(self, v: Any) -> str:
         self.__check()
-        v = b64e(EasyRSA(public_key=self.__key_pair["public_key"]).encrypt(jd(v)))
+        k = self.__skey
+        v = self.__aese(k, jd(v))
+        k = self.__rsae(k)
         hash = self.__hmac(v)
-        return f"{hash} {v}"
+        return f"{hash} {k} {v}"
 
     def set(self, id: str, pw: str, k: str, v: Any) -> Any:
         Broker(app_name=self.__app_name, username=self.__username(id, pw)).set(self.__hmac(k), self.__encrypt(v))
@@ -34,9 +39,9 @@ class Agent(object):
 
     def __decrypt(self, v: str) -> Any:
         self.__check()
-        hash, v = v.split(" ")
+        hash, k, v = v.split(" ")
         if hash == self.__hmac(v):
-            return jl(EasyRSA(private_key=self.__key_pair["private_key"]).decrypt(b64d(v)))
+            return jl(self.__aesd(k, v))
         raise Exception("credentials are tampered due to different hmac")
 
     def get(self, id: str, pw: str, k: str) -> Any:
